from dataclasses import dataclass


class RequestMethod(object):
    METHOD = None


class RequestPOST(RequestMethod):
    METHOD = 'post'


class RequestGET(RequestMethod):
    METHOD = 'get'


@dataclass
class RequestMethods(object):
    POST = RequestPOST()
    GET = RequestGET()



import requests

from mono.core.api import MonoApi
from mono.request import RequestMethods


__all__ = (
    'MonoMerchant',
)


class MonoMerchant(MonoApi):
    MERCHANT_INFO = MonoApi.BASE_URL + '/api/merchant/details'

    def get_merchant_info(self) -> requests.Response:
        """
        Returns information about the merchant

        :return: requests.Response
        """

        return self.request(
            url=self.MERCHANT_INFO,
            method=RequestMethods.GET
        )



import requests

from mono.core.api import MonoApi
from mono.request import RequestMethods

from .schemas import InvoiceData, StatusInvoiceData


class MonoInvoice(MonoApi):
    CREATE = MonoApi.BASE_URL + '/api/merchant/invoice/create'
    STATUS = MonoApi.BASE_URL + '/api/merchant/invoice/status'

    def __init__(self, token: str):
        super().__init__(token=token)

    def create_invoice(self, invoice_data: InvoiceData) -> requests.Response:
        """
        Create invoice for payment

        :param invoice_data:
        :return: requests.Response
        """

        return self.request(
            url=self.CREATE,
            method=RequestMethods.POST,
            data=invoice_data.as_dict()
        )

    def invoice_status(self, invoice_data: StatusInvoiceData) -> requests.Response:
        """
        Method of checking the status of the account when the seller is out of sync or there is no webHookUrl
        when creating the account

        :param invoice_data:
        :return: requests.Response
        """

        return self.request(
            url=self.STATUS,
            method=RequestMethods.GET,
            params=invoice_data.as_dict()
        )



from dataclasses import dataclass

from mono.schemas import BaseDataclass
from mono.utils.currencies import Currencies


__all__ = (
    'InvoiceData',
    'StatusInvoiceData'
)


@dataclass
class InvoiceData(BaseDataclass):
    amount: int | float
    ccy: int = Currencies.UAH  # Currency. Default: UAH
    redirectUrl: str = ''
    webHookUrl: str = ''
    validity: int = 86400  # Invoice alive seconds. Default: 24 hour
    paymentType: str = 'debit'  # Payment type. Enum: "debit" "hold". Default: 'debit'
    qrId: str = ''  # QR cashier ID to set the payment amount on existing QR cashiers


@dataclass
class StatusInvoiceData(BaseDataclass):
    invoiceId: str



import requests

from mono.request import RequestPOST, RequestGET, exceptions


class MonoApi(object):
    BASE_URL = 'https://api.monobank.ua'
    AUTH_HEADER = 'X-Token'
    TOKEN = None

    def __init__(self, token: str):
        self.TOKEN = token

    @staticmethod
    def post(
            url: str,
            data: dict,
            headers: dict = None
    ) -> requests.Response:
        """
        Sends a POST request through the requests.post method and returns a response to the server.

        Throws errors:
            * TypeError: if any parameter does not match the correct type

        :param url: string, request url
        :param data: dict, request data
        :param headers: dict, request headers
        :return: requests.Response
        """

        if not isinstance(url, str):
            raise TypeError(f'url arg must be str not `{type(url)}`')

        if headers and not isinstance(headers, dict):
            raise TypeError(f'headers arg must be dict not `{type(headers)}`')

        if not isinstance(data, dict):
            raise TypeError(f'data arg must be dict not `{type(data)}`')

        return requests.post(
            url=url,
            json=data,
            headers=headers
        )

    @staticmethod
    def get(
            url: str,
            params: dict = None,
            headers: dict = None
    ) -> requests.Response:
        """
        Sends a GET request through the requests.get method and returns a response to the server.

        Throws errors:
            * TypeError: if any parameter does not match the correct type

        :param url: string, request url
        :param params: dict, request params
        :param headers: dict, request headers
        :return: requests.Response
        """

        if not isinstance(url, str):
            raise TypeError(f'url arg must be str not `{type(url)}`')

        if params and not isinstance(params, dict):
            raise TypeError(f'params arg must be dict not `{type(params)}`')

        if headers and not isinstance(headers, dict):
            raise TypeError(f'headers arg must be dict not `{type(headers)}`')

        return requests.get(
            url=url,
            params=params,
            headers=headers
        )

    def request(
            self,
            url: str,
            method: RequestPOST | RequestGET,
            data: dict = None,
            params: dict = None,
            headers: dict = None
    ) -> requests.Response:
        """
        Sends a request to the server depending on the method and url and returns a response to the server.

        Throws errors:
            * TypeError: if any parameter does not match the correct type
            * RequestMethodNotFoundError: if the method parameter does not match the available class object

        :param url: string, request url
        :param method: instance of RequestPOST or RequestGET classes
        :param data: dict, request data with POST request
        :param params: dict, request data with GET request
        :param headers: dict, request headers
        :return: requests.Response
        """

        request_headers = {self.AUTH_HEADER: self.TOKEN}

        if headers:
            if not isinstance(headers, dict):
                raise TypeError(f'headers arg must be dict not `{type(headers)}`')
            request_headers = {**request_headers, **headers}

        if isinstance(method, RequestPOST):
            return self.post(url=url, data=data, headers=request_headers)
        elif isinstance(method, RequestGET):
            return self.get(url=url, params=params, headers=request_headers)
        else:
            raise exceptions.RequestMethodNotFoundError(f'method {method} not found to processing')
